@@ -1,23 +1,25 @@
-from abc import abstractclassmethod
-from ._training import Trainer, StopTrainingError, Events
 import numpy as np
+from ._training import Trainer, StopTrainingError, Events
 
 class Callback():
     def __init__(self) -> None:
         pass
     
-    @abstractclassmethod
     def get_callbacks_dict(self):
-        pass
+        return {i: getattr(self, i.value) for i in Events if hasattr(self, i.value)}
+        
 
-def guess_auto_mode(monitor, mode) -> str:
+def _guess_auto_mode(monitor, mode) -> str:
     if mode != 'auto':
-        assert mode == 'max' or mode == 'min'
+        assert mode in ('max', 'min')
         return mode
     for s in ['loss', 'mse', 'mae', 'mape', 'error', 'err']:
         if s in monitor:
             return 'min'
-    return 'max'
+    for s in ['acc', 'auc', 'score']:
+        if s in monitor:
+            return 'max'
+    raise ValueError("Unknown monitor. Please specify `mode` for it manually (either 'min' or 'max').")
 
 class ModelCheckpoint(Callback):
     def __init__(self, filepath, monitor='val_loss', mode='auto', save_best_only=True, save_weights_only=True, verbose=0):
@@ -28,7 +30,7 @@ class ModelCheckpoint(Callback):
 
         self.filepath = filepath
         self.monitor = monitor
-        self.mode = guess_auto_mode(monitor, mode)
+        self.mode = _guess_auto_mode(monitor, mode)
         self.save_best_only = save_best_only
         self.save_weights_only = save_weights_only
         self.verbose = verbose
@@ -38,11 +40,7 @@ class ModelCheckpoint(Callback):
     def is_better(self, curr_score):
         if self.mode == 'max':
             return curr_score > self.best_score
-        else:   # mode == 'min'
-            return curr_score < self.best_score
-
-    def get_callbacks_dict(self):
-        return {Events.ON_EPOCH_END: self.on_epoch_end}
+        return curr_score < self.best_score
 
     def on_epoch_end(self, trainer: Trainer):
         filename = self.filepath.format(epoch=trainer.epoch, **trainer.logger.metrics)
@@ -67,7 +65,7 @@ class EarlyStopping(Callback):
         super(EarlyStopping, self).__init__()
 
         self.monitor = monitor
-        self.mode = guess_auto_mode(monitor, mode)
+        self.mode = _guess_auto_mode(monitor, mode)
         self.min_delta = min_delta
         self.chances = self.patience = patience
         self.baseline = baseline
@@ -79,13 +77,12 @@ class EarlyStopping(Callback):
     def is_better(self, curr_score):
         if self.mode == 'max':
             return curr_score - self.best_score > self.min_delta
-        else:   # mode == 'min'
-            if self.baseline != None and curr_score > self.baseline:
-                return False
-            return self.best_score - curr_score > self.min_delta
+        if self.baseline is not None and curr_score > self.baseline:
+            return False
+        return self.best_score - curr_score > self.min_delta
 
     def is_surpass_baseline(self, curr_score):
-        if self.baseline == None or self.baseline_flag:
+        if self.baseline is None or self.baseline_flag:
             return True
         if self.mode == 'max' and curr_score > self.baseline:
             self.baseline_flag = True
@@ -95,9 +92,6 @@ class EarlyStopping(Callback):
             return True
         return False
 
-    def get_callbacks_dict(self):
-        return {Events.ON_EPOCH_END: self.on_epoch_end}
-
     def on_epoch_end(self, trainer: Trainer):
         if self.monitor not in trainer.logger.metrics:
             raise KeyError(f'No such metric: {self.monitor}.')
@@ -119,22 +113,24 @@ class EarlyStopping(Callback):
 
 
 
+from torch.optim.lr_scheduler import ReduceLROnPlateau
 
 class LRScheduler(Callback):
     def __init__(self, lr_scheduler) -> None:
         super(LRScheduler, self).__init__()
         self.lr_scheduler = lr_scheduler
 
-    def get_callbacks_dict(self):
-        return {Events.ON_EPOCH_END: self.on_epoch_end}
-
-    def on_epoch_end(self, trainer):
-        self.lr_scheduler.step()
+    def on_epoch_end(self, trainer: Trainer):
+        if isinstance(self.lr_scheduler, ReduceLROnPlateau):
+            self.lr_scheduler.step(trainer.logger.metrics['val_loss'])
+        else:
+            self.lr_scheduler.step()
+        
 
 
 
 class LambdaCallback(Callback):
-    def __init__(self, on_epoch_begin=None, on_epoch_end=None, on_train_begin=None, on_train_end=None) -> None:
+    def __init__(self, on_epoch_begin=None, on_epoch_end=None, on_batch_begin=None, on_batch_end=None, on_train_begin=None, on_train_end=None) -> None:
         super(LambdaCallback, self).__init__()
 
         self.callbacks_dict = {}
@@ -147,6 +143,10 @@ class LambdaCallback(Callback):
             self.callbacks_dict.update({Events.ON_TRAIN_BEGIN: on_train_begin})
         if on_train_end:
             self.callbacks_dict.update({Events.ON_TRAIN_END: on_train_end})
+        if on_batch_begin:
+            self.callbacks_dict.update({Events.ON_BATCH_BEGIN: on_batch_begin})
+        if on_batch_end:
+            self.callbacks_dict.update({Events.ON_BATCH_END: on_batch_end})
 
     def get_callbacks_dict(self):
         return self.callbacks_dict
